/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.location.fence.impl;

import com.yiring.app.domain.location.FenceAlarmPushLog;
import com.yiring.app.domain.location.FenceAlarmPushLogRepository;
import com.yiring.app.param.location.fence.FenceAlarmPushLogConditionParam;
import com.yiring.app.service.location.fence.FenceAlarmPushLogService;
import com.yiring.app.vo.location.fence.FenceAlarmPushLogVo;
import com.yiring.auth.domain.user.User;
import com.yiring.common.core.Result;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/18 10:27
 */
@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class FenceAlarmPushLogServiceImpl implements FenceAlarmPushLogService {

    @Resource
    private FenceAlarmPushLogRepository fenceAlarmPushLogRepository;

    @Override
    public Result<PageVo<FenceAlarmPushLogVo>> findList(
        FenceAlarmPushLogConditionParam conditionParam,
        PageParam pageParam
    ) {
        Pageable pageable = PageParam.toPageable(pageParam);
        Specification<FenceAlarmPushLog> specification = (root, query, criteriaBuilder) -> {
            ArrayList<Predicate> list = new ArrayList<>();
            if (conditionParam.getFenceAlarmId() != null) {
                list.add(
                    criteriaBuilder.equal(
                        root.get(FenceAlarmPushLog.Fields.fenceAlarm),
                        conditionParam.getFenceAlarmId()
                    )
                );
            }
            if (conditionParam.getInformManner() != null) {
                CriteriaBuilder.In<Object> in = criteriaBuilder.in(root.get(FenceAlarmPushLog.Fields.informManner));
                for (Integer item : conditionParam.getInformManner()) {
                    in.value(item);
                }
                list.add(in);
            }
            if (conditionParam.getStatus() != null) {
                list.add(criteriaBuilder.equal(root.get(FenceAlarmPushLog.Fields.status), conditionParam.getStatus()));
            }
            if (conditionParam.getPhone() != null) {
                Join<Object, Object> join = root.join(FenceAlarmPushLog.Fields.user, JoinType.LEFT);
                list.add(criteriaBuilder.like(join.get(User.Fields.mobile), "%" + conditionParam.getPhone() + "%"));
            }
            Predicate[] array = list.toArray(new Predicate[0]);
            return criteriaBuilder.and(array);
        };
        Page<FenceAlarmPushLog> page = fenceAlarmPushLogRepository.findAll(specification, pageable);
        List<FenceAlarmPushLogVo> voList = page.get().map(FenceAlarmPushLogVo::transform).collect(Collectors.toList());
        PageVo<FenceAlarmPushLogVo> pageVo = PageVo.build(voList, voList.size());
        return Result.ok(pageVo);
    }
}
