/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.location.tag.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.github.liaochong.myexcel.core.SaxExcelReader;
import com.yiring.app.domain.location.LocationTag;
import com.yiring.app.domain.location.LocationTagRepository;
import com.yiring.app.domain.visitor.Car;
import com.yiring.app.excel.location.tag.LocationTagExportExcel;
import com.yiring.app.excel.location.tag.LocationTagImportExcel;
import com.yiring.app.feign.zy.location.tag.LocationTagClient;
import com.yiring.app.param.location.tag.LocationTagAddParam;
import com.yiring.app.param.location.tag.LocationTagDeleteParam;
import com.yiring.app.param.location.tag.LocationTagFindParam;
import com.yiring.app.param.zy.location.tag.ZyLocationTagAddParam;
import com.yiring.app.service.location.tag.LocationTagService;
import com.yiring.app.util.zy.ZyUtil;
import com.yiring.app.vo.location.tag.LocationTagIndexVo;
import com.yiring.app.vo.location.tag.LocationTagVo;
import com.yiring.app.vo.user.UserVo;
import com.yiring.app.vo.visitor.CarVo;
import com.yiring.auth.domain.user.User;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.IndexParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

/**
 * 定位标签
 * @author LJ-2204
 * @date 2022/4/14
 */

@Transactional(rollbackFor = RuntimeException.class)
@Service
@Slf4j
public class LocationTagServiceImpl implements LocationTagService {

    final Integer FACTORY_ID = 100;

    @Resource
    LocationTagRepository locationTagRepository;

    @Resource
    LocationTagClient locationTagClient;

    @Override
    public Result<String> addLocationTag(LocationTagAddParam locationTagAddParam) {
        if (hasLocationTagInfoByCode(locationTagAddParam.getCode())) return Result.no(Status.BAD_REQUEST, "编号已存在");

        LocationTag locationTag = LocationTagAddParam.transform(locationTagAddParam);
        locationTag = locationTagRepository.save(locationTag);
        ZyLocationTagAddParam zyLocationTagAddParam = ZyLocationTagAddParam.transform(locationTag, FACTORY_ID);
        JSONObject jsonObject = locationTagClient.tagSave(
            JSONUtil.toJsonStr(zyLocationTagAddParam),
            "Bearer " + ZyUtil.manageLogin()
        );
        if (!ObjectUtil.equals(jsonObject.get("code"), 200)) {
            throw new RuntimeException(StrUtil.toString(jsonObject.get("msg")));
        }
        HashMap<String, Object> page = new HashMap<>();
        page.put("orgId", FACTORY_ID);
        page.put("pageNum", 1);
        page.put("pageSize", 1);
        page.put("tagId", locationTag.getCode());
        JSONObject result = locationTagClient.tagPage(JSONUtil.toJsonStr(page), ZyUtil.manageLogin());
        if (!ObjectUtil.equals(jsonObject.get("code"), 200)) {
            throw new RuntimeException(StrUtil.toString(jsonObject.get("msg")));
        }
        JSONObject data = JSONUtil.parseObj(result.get("data"));
        JSONArray records = JSONUtil.parseArray(data.get("records"));

        List<IdParam> idParams = JSONUtil.toList(JSONUtil.toJsonStr(records), IdParam.class);
        Long id = idParams.get(0).getId();
        locationTag.setLinkId(id);
        locationTagRepository.save(locationTag);
        return Result.ok();
    }

    @Override
    public Result<PageVo<LocationTagVo>> findLocationTagPage(LocationTagFindParam param, PageParam pageParam) {
        Pageable pageable = PageRequest.of(pageParam.getPageNo() - 1, pageParam.getPageSize());
        Page<LocationTag> locationTags = locationTagRepository.findAll(condition(param), pageable);
        List<LocationTagVo> locationTagVos = locationTags
            .stream()
            .map(LocationTagVo::transform)
            .collect(Collectors.toList());
        PageVo<LocationTagVo> page = PageVo.build(locationTagVos, locationTags.getTotalElements());
        return Result.ok(page);
    }

    @Override
    public Result<String> deleteLocationTag(LocationTagDeleteParam locationTagDeleteParam) {
        Map<Object, Object> map = MapUtil.createMap(HashMap.class);
        map.put("orgId", FACTORY_ID);
        Long[] ids = locationTagDeleteParam.getIds();

        for (Long id : ids) {
            Optional<LocationTag> locationTagOptional = locationTagRepository.findOne(
                Example.of(LocationTag.builder().id(id).build())
            );

            if (locationTagOptional.isEmpty()) {
                throw new RuntimeException();
            }

            LocationTag locationTag = locationTagOptional.get();
            locationTagRepository.deleteById(id);
            Object code = locationTagClient
                .tagDelete(Convert.toStr(locationTag.getLinkId()), JSONUtil.toJsonStr(map), ZyUtil.manageLogin())
                .get("code");
            if (!ObjectUtil.equals(code, 200)) {
                throw new RuntimeException();
            }
        }
        return Result.ok();
    }

    @Override
    public void exportLocationTagInfo(LocationTagFindParam param, HttpServletResponse response) {
        List<LocationTag> locationTags = locationTagRepository.findAll(condition(param));
        List<LocationTagExportExcel> locationTagExportExcels = locationTags
            .stream()
            .map(LocationTagExportExcel::transform)
            .collect(Collectors.toList());
        try (
            DefaultStreamExcelBuilder<LocationTagExportExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(LocationTagExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(locationTagExportExcels);

            String fileName = URLEncoder.encode("定位标签信息.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出定位信息失败: " + e.getMessage());
        }
    }

    @Override
    public Result<String> importLocationTagInfo(MultipartFile file) {
        if (file.isEmpty()) throw new RuntimeException("请选择文件");
        List<LocationTagImportExcel> locationTagImportExcels;
        try (InputStream inputStream = file.getInputStream()) {
            locationTagImportExcels =
                SaxExcelReader
                    .of(LocationTagImportExcel.class)
                    .rowFilter(row -> row.getRowNum() > 0)
                    .ignoreBlankRow()
                    .read(inputStream);
        } catch (IOException e) {
            log.info(e.getMessage());
            throw new RuntimeException("文件导入异常");
        }
        List<LocationTag> locationTags;

        locationTags =
            locationTagImportExcels
                .stream()
                .filter(locationTagImportExcel -> !hasLocationTagInfoByCode(locationTagImportExcel.getCode()))
                .map(LocationTagImportExcel::transform)
                .collect(Collectors.toList());

        locationTags.forEach(locationTag -> {
            locationTag = locationTagRepository.save(locationTag);
            ZyLocationTagAddParam zyLocationTagAddParam = ZyLocationTagAddParam.transform(locationTag, FACTORY_ID);
            JSONObject jsonObject = locationTagClient.tagSave(
                JSONUtil.toJsonStr(zyLocationTagAddParam),
                "Bearer " + ZyUtil.manageLogin()
            );
            if (!ObjectUtil.equals(jsonObject.get("code"), 200)) {
                throw new RuntimeException(StrUtil.toString(jsonObject.get("msg")));
            }
            HashMap<String, Object> page = new HashMap<>();
            page.put("orgId", FACTORY_ID);
            page.put("pageNum", 1);
            page.put("pageSize", 1);
            page.put("tagId", locationTag.getCode());
            JSONObject result = locationTagClient.tagPage(JSONUtil.toJsonStr(page), ZyUtil.manageLogin());
            if (!ObjectUtil.equals(jsonObject.get("code"), 200)) {
                throw new RuntimeException(StrUtil.toString(jsonObject.get("msg")));
            }
            JSONObject data = JSONUtil.parseObj(result.get("data"));
            JSONArray records = JSONUtil.parseArray(data.get("records"));

            List<IdParam> idParams = JSONUtil.toList(JSONUtil.toJsonStr(records), IdParam.class);
            Long id = idParams.get(0).getId();
            locationTag.setLinkId(id);
            locationTagRepository.save(locationTag);
        });
        return Result.ok();
    }

    @Override
    public Result<PageVo<LocationTagIndexVo>> indexLocationTag(IndexParam indexParam) {
        Specification<LocationTag> specification = (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();
            if (StrUtil.isNotEmpty(indexParam.getStr())) {
                predicates.add(cb.like(root.get(LocationTag.Fields.code), "%" + indexParam.getStr() + "%"));
            }

            Order order = cb.desc(root.get(BasicEntity.Fields.createTime));
            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };

        List<LocationTag> locationTags = locationTagRepository.findAll(specification);
        List<LocationTagIndexVo> locationTagIndexVos = locationTags
            .stream()
            .map(LocationTagIndexVo::transform)
            .collect(Collectors.toList());
        PageVo<LocationTagIndexVo> pageVo = PageVo.build(locationTagIndexVos, locationTagIndexVos.size());
        return Result.ok(pageVo);
    }

    @Override
    public Result<PageVo<LocationTagVo>> findLocationTag(LocationTagFindParam param) {
        List<LocationTag> locationTags = locationTagRepository.findAll(condition(param));
        List<LocationTagVo> locationTagVos = locationTags
            .stream()
            .map(locationTag -> {
                LocationTagVo locationTagVo = LocationTagVo.transform(locationTag);
                if (ObjectUtil.isNotEmpty(locationTag.getUser())) {
                    User user = locationTag.getUser();
                    UserVo userVo = UserVo.builder().id(user.getId()).realName(user.getRealName()).build();
                    locationTagVo.setUserVo(userVo);
                } else if (ObjectUtil.isNotEmpty(locationTag.getCar())) {
                    Car car = locationTag.getCar();
                    CarVo carVo = CarVo.builder().id(car.getId()).carNum(car.getCarNum()).build();
                    locationTagVo.setCarVo(carVo);
                }
                return locationTagVo;
            })
            .collect(Collectors.toList());
        PageVo<LocationTagVo> page = PageVo.build(locationTagVos, locationTagVos.size());
        return Result.ok(page);
    }

    @Override
    public Result<LocationTagVo> info(LocationTagFindParam param) {
        Specification<LocationTag> specification = (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (StrUtil.isNotEmpty(param.getCode())) {
                predicates.add(cb.equal(root.get(LocationTag.Fields.code), param.getCode()));
            }

            return cq.where(predicates.toArray(new Predicate[0])).getRestriction();
        };

        Optional<LocationTag> locationTagOptional = locationTagRepository.findOne(specification);

        if (locationTagOptional.isEmpty()) {
            return Result.no(Status.BAD_REQUEST);
        }

        LocationTag locationTag = locationTagOptional.get();

        LocationTagVo locationTagVo = LocationTagVo.transform(locationTag);
        if (ObjectUtil.isNotEmpty(locationTag.getUser())) {
            User user = locationTag.getUser();
            UserVo userVo = UserVo.builder().id(user.getId()).realName(user.getRealName()).build();
            locationTagVo.setUserVo(userVo);
        } else if (ObjectUtil.isNotEmpty(locationTag.getCar())) {
            Car car = locationTag.getCar();
            CarVo carVo = CarVo.builder().id(car.getId()).carNum(car.getCarNum()).build();
            locationTagVo.setCarVo(carVo);
        }
        return Result.ok(locationTagVo);
    }

    private boolean hasLocationTagInfoByCode(String code) {
        LocationTag locationTag = LocationTag.builder().code(code).build();
        return locationTagRepository.count(Example.of(locationTag)) > 0;
    }

    private Specification<LocationTag> condition(LocationTagFindParam param) {
        return (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (StrUtil.isNotEmpty(param.getCode())) {
                predicates.add(cb.like(root.get(LocationTag.Fields.code), "%" + param.getCode() + "%"));
            }

            if (ObjectUtil.isNotEmpty(param.getType())) {
                predicates.add(cb.equal(root.get(LocationTag.Fields.type), param.getType()));
            }

            if (ObjectUtil.isNotEmpty(param.getSilent())) {
                predicates.add(cb.equal(root.get(LocationTag.Fields.silent), param.getSilent()));
            }

            if (ObjectUtil.isNotEmpty(param.getUsed())) {
                predicates.add(cb.equal(root.get(LocationTag.Fields.used), param.getUsed()));
            }

            if (ObjectUtil.isNotEmpty(param.getOut())) {
                predicates.add(cb.equal(root.get(LocationTag.Fields.out), param.getOut()));
            }

            Order order = cb.desc(root.get(BasicEntity.Fields.createTime));

            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };
    }
}
