/* (C) 2022 YiRing, Inc. */
package com.yiring.app.web.location.fence;

import com.yiring.app.param.location.fence.FenceAlarmConditionParam;
import com.yiring.app.service.location.fence.FenceAlarmService;
import com.yiring.app.vo.IdNameVo;
import com.yiring.app.vo.location.fence.FenceAlarmVo;
import com.yiring.common.core.Result;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.Objects;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import org.springframework.data.domain.Sort;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/17 14:41
 */
@Api(tags = "围栏报警数据管理")
@Validated
@RestController
@RequestMapping("/fence/alarm")
public class FenceAlarmController {

    @Resource
    private FenceAlarmService fenceAlarmService;

    @ApiOperation("查询围栏报警数据列表")
    @GetMapping("/findList")
    public Result<PageVo<FenceAlarmVo>> findList(@Valid FenceAlarmConditionParam param, @Valid PageParam pageParam) {
        if (Objects.isNull(pageParam.getSortField())) {
            pageParam.setSortField(BasicEntity.Fields.createTime);
            pageParam.setSortOrder(Sort.Direction.DESC);
        }
        return fenceAlarmService.findList(param, pageParam);
    }

    @ApiOperation("查询围栏下拉框")
    @GetMapping("/findFenceList")
    public Result<PageVo<IdNameVo>> findFenceList() {
        return fenceAlarmService.findFenceList();
    }

    @ApiOperation(value = "导出围栏报警数据", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    @GetMapping("/export")
    public void export(@Valid FenceAlarmConditionParam param, HttpServletResponse response) {
        fenceAlarmService.export(param, response);
    }
}
